// ********************************************************************
// USART - Grundfunktionen 
// AVR-Version / GGC 4.3.2
// Version 1.0 / Stand: 20.10.2010
// Autor: Matthias Kahnt
// ********************************************************************
// Die Programmfunktionen drfen fr private Nutzung verwendet und
// verndert werden. Fr gewerbliche Nutzung ist die Zustimmung
// des Autors erforderlich.
// Kontakt: Matthias Kahnt, e-mail: az51@gmx.net
// ********************************************************************
// Die Nutzung erfolgt auf eigene Gefahr.
// Fr eventuell entstehende Schden wird keine Haftung bernommen.
// ********************************************************************
//
//************************************************************************
// AVR/GCC LIBS
//************************************************************************
#include "defines.h"            // Projektdefinitionen
#include <avr/io.h>             // IO-Grundfunktionen
#include <avr/interrupt.h>      // Interrupt-Funktionen
#include <stdio.h>              // Standardfunktion

#include "usart_avr.h"          // Headerfile
#include "ULOG_AVR_Projekt.h"

// ***********************************************************************
// USART - Initialisierung
// bergabe: CPU-Taktfrequenz
// ***********************************************************************
void usart_init(unsigned long taktfrequenz) {
unsigned long baudrate;
unsigned int ubr_temp;
  // UART0 (PIN3) Baudrate
  switch (awd.uart0_baudrate) {
    case 0:  baudrate =    150; break;
    case 1:  baudrate =    300; break;
    case 2:  baudrate =    600; break;
    case 3:  baudrate =   1200; break;
    case 4:  baudrate =   2400; break;
    case 5:  baudrate =   4800; break;
    case 6:  baudrate =   9600; break;
    case 7:  baudrate =  14400; break;
    case 8:  baudrate =  19200; break;
    case 9:  baudrate =  38400; break;
    case 10: baudrate =  57600; break;
    case 11: baudrate =  76800; break;
    case 12: baudrate = 115200; break;
    case 13: baudrate = 230400; break;
    default: baudrate =   9600;
  }
  // Baudrate berechnen und setzen
  ubr_temp = taktfrequenz / (baudrate * 16L) - 1;
  UBRR0L   = ubr_temp % 256;
  UBRR0H   = ubr_temp / 256;

  // UART1 (PIN2) Baudrate
  switch (awd.uart1_baudrate) {
    case 0:  baudrate =    150; break;
    case 1:  baudrate =    300; break;
    case 2:  baudrate =    600; break;
    case 3:  baudrate =   1200; break;
    case 4:  baudrate =   2400; break;
    case 5:  baudrate =   4800; break;
    case 6:  baudrate =   9600; break;
    case 7:  baudrate =  14400; break;
    case 8:  baudrate =  19200; break;
    case 9:  baudrate =  38400; break;
    case 10: baudrate =  57600; break;
    case 11: baudrate =  76800; break;
    case 12: baudrate = 115200; break;
    case 13: baudrate = 230400; break;
    default: baudrate =   9600;
  }
  // Baudrate berechnen und setzen
  ubr_temp = taktfrequenz / (baudrate * 16L) - 1;
  UBRR1L   = ubr_temp % 256;
  UBRR1H   = ubr_temp / 256;

  // UART0 (PIN3) Paritt
  UCSR0C = 0; // UMSELn1 UMSELn0 UPMn1 UPMn0 USBSn UCSZn1 UCSZn0 UCPOLn
  switch(awd.uart0_parity) {
    case  0: break;                                             // keine (UPM01=0 UPM00=0)
    case  1: UCSR0C |= ((1 << UPM01)); break;                   // EVEN  (UPM01=1 UPM00=0)
    case  2: UCSR0C |= ((1 << UPM01) | (1 << UPM00)); break;    // ODD   (UPM01=1 UPM00=1)
  }
  // UART1 (PIN2) Paritt
  UCSR1C = 0; // UMSELn1 UMSELn0 UPMn1 UPMn0 USBSn UCSZn1 UCSZn0 UCPOLn
  switch(awd.uart1_parity) {
    case  0: break;                                             // keine (UPM11=0 UPM00=0)
    case  1: UCSR1C |= ((1 << UPM11)); break;                   // EVEN  (UPM11=1 UPM10=0)
    case  2: UCSR1C |= ((1 << UPM11) | (1 << UPM10)); break;    // ODD   (UPM11=1 UPM10=1)
  }

  // UART0 (PIN3) Datenbits
  UCSR0B = 0; // RXCIEn TXCIEn UDRIEn RXENn TXENn UCSZn2 RXB8n TXB8n
  switch(awd.uart0_datenbits) {
    case  0: break;                                             // 5 (UCSZ02=0 UCSZ01=0 UCSZ00=0)
    case  1: UCSR0C |= ((1 << UCSZ00)); break;                  // 6 (UCSZ02=0 UCSZ01=0 UCSZ00=1)
    case  2: UCSR0C |= ((1 << UCSZ01)); break;                  // 7 (UCSZ02=0 UCSZ01=1 UCSZ00=0)
    case  3: UCSR0C |= ((1 << UCSZ01) | (1 << UCSZ00)); break;  // 8 (UCSZ02=0 UCSZ01=1 UCSZ00=1)
    case  4: UCSR0C |= ((1 << UCSZ01) | (1 << UCSZ00));         // 9 (UCSZ02=1 UCSZ01=1 UCSZ00=1)
             UCSR0B |= ((1 << UCSZ02));
             break;
  }
  // UART1 (PIN2) Datenbits
  UCSR1B = 0; // RXCIEn TXCIEn UDRIEn RXENn TXENn UCSZn2 RXB8n TXB8n
  switch(awd.uart1_datenbits) {
    case  0: break;                                             // 5 (UCSZ12=0 UCSZ11=0 UCSZ10=0)
    case  1: UCSR1C |= ((1 << UCSZ10)); break;                  // 6 (UCSZ12=0 UCSZ11=0 UCSZ10=1)
    case  2: UCSR1C |= ((1 << UCSZ11)); break;                  // 7 (UCSZ12=0 UCSZ11=1 UCSZ10=0)
    case  3: UCSR1C |= ((1 << UCSZ11) | (1 << UCSZ10)); break;  // 8 (UCSZ12=0 UCSZ11=1 UCSZ10=1)
    case  4: UCSR1C |= ((1 << UCSZ11) | (1 << UCSZ10));         // 9 (UCSZ12=1 UCSZ11=1 UCSZ10=1)
             UCSR1B |= ((1 << UCSZ12));
             break;
  }

  // UART 0 und 1 freigeben fr Empfang
  UCSR0B =((1 << RXEN0) | (1<< RXCIE0));
  UCSR1B =((1 << RXEN1) | (1<< RXCIE1));
}

// ********************************************************************
// UART Interruptroutine
// UART0 (D-SUB PIN 3)
// ********************************************************************
ISR(USART0_RX_vect) { 
unsigned char temp;
  temp = UDR0;  // UDR0 immer abholen, sonst Fehler bei Buffer voll
  if (logset.enable) {
    switch (awd.uart_log_mode) {
        case 0: // Log-Mode 0
            logbuch.mode0[logset.index].rxd_pin3 = temp;        // Datenbyte
            logbuch.mode0[logset.index].rxd_pin2 = 0;           // nicht verwendet
            logbuch.mode0[logset.index].timecode = time_ms;     // Timecode
            logbuch.mode0[logset.index].status   = PIND & 0xF8; // Status
            logbuch.mode0[logset.index].status  |= 0x01;        // Kennung fr Pin3 gltig
            break;
        case 1: // Log-Mode 1
            logbuch.mode1[logset.index].rxd_pin3 = temp;        // Datenbyte
            logbuch.mode1[logset.index].rxd_pin2 = 0;           // nicht verwendet
            logbuch.mode1[logset.index].status   = PIND & 0xF8; // Status
            logbuch.mode1[logset.index].status  |= 0x01;        // Kennung fr Pin3 gltig
            break;
        case 2: // Log-Mode 2
            logbuch.mode2[logset.index].rxd_pin3 = temp;        // Datenbyte
            logbuch.mode2[logset.index].rxd_pin2 = 0;           // nicht verwendet
            logbuch.mode2[logset.index].timecode = time_ms;     // Timecode
            break;
        case 3: // Log-Mode 3
            logbuch.mode3[logset.index].rxd_pin3 = temp;        // Datenbyte
            logbuch.mode3[logset.index].rxd_pin2 = 0;           // nicht verwendet
            break;
        case 4: // Log-Mode 4
            break;
        case 5: // Log-Mode 5
            logbuch.mode5[logset.index].rxd_pin3 = temp;        // Datenbyte
            break;
    }
    // Prfung auf Bufferberlauf
    if (logset.index < (logset.max - 1)) {
      logset.index++;
    } else {
      logset.enable = 0;
    }
  }
}

// ********************************************************************
// UART Interruptroutine
// UART1 (D-SUB PIN 2)
// ********************************************************************
ISR(USART1_RX_vect) { 
unsigned char temp;
  temp = UDR1;  // UDR1 immer abholen, sonst Fehler bei Buffer voll
  if (logset.enable) {
    switch (awd.uart_log_mode) {
        case 0: // Log-Mode 0
            logbuch.mode0[logset.index].rxd_pin3 = 0;           // nicht verwendet
            logbuch.mode0[logset.index].rxd_pin2 = temp;        // Datenbyte
            logbuch.mode0[logset.index].timecode = time_ms;     // Timecode
            logbuch.mode0[logset.index].status   = PIND & 0xF8; // Status
            logbuch.mode0[logset.index].status  |= 0x02;        // Kennung fr Pin2 gltig
            break;
        case 1: // Log-Mode 1
            logbuch.mode1[logset.index].rxd_pin3 = 0;           // nicht verwendet
            logbuch.mode1[logset.index].rxd_pin2 = temp;        // Datenbyte
            logbuch.mode1[logset.index].status   = PIND & 0xF8; // Status
            logbuch.mode1[logset.index].status  |= 0x02;        // Kennung fr Pin2 gltig
            break;
        case 2: // Log-Mode 2
            logbuch.mode2[logset.index].rxd_pin3 = 0;           // nicht verwendet
            logbuch.mode2[logset.index].rxd_pin2 = temp;        // Datenbyte
            logbuch.mode2[logset.index].timecode = time_ms;     // Timecode
            break;
        case 3: // Log-Mode 3
            logbuch.mode3[logset.index].rxd_pin3 = 0;           // nicht verwendet
            logbuch.mode3[logset.index].rxd_pin2 = temp;        // Datenbyte
            break;
        case 4: // Log-Mode 4
            logbuch.mode4[logset.index].rxd_pin2 = temp;        // Datenbyte
            break;
        case 5: // Log-Mode 5
            break;
    }
    // Prfung auf Bufferberlauf
    if (logset.index < (logset.max - 1)) {
      logset.index++;
    } else {
      logset.enable = 0;
    }
  }
}
